/*******************************************************************************
    Copyright (c) 2016 NVIDIA Corporation

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

        The above copyright notice and this permission notice shall be
        included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
    THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
    DEALINGS IN THE SOFTWARE.

    Authors: Javier Cabezas (jcabezas@nvidia.com)

*******************************************************************************/

#ifndef _UVM_PERF_H
#define _UVM_PERF_H

#include "uvm_linux_ioctl.h"

// This enum contains the main sub-modules that handle performance optimizations in UVM.
//
// UVM_PERF_MODULE_TYPE_STATS: collects performance events and statistics (only used in debug mode)
// UVM_PERF_MODULE_TYPE_THRASHING: detects memory thrashing scenarios and provides thrashing prevention mechanisms
// UVM_PERF_MODULE_TYPE_PREFETCH: detects memory prefetching opportunities
typedef enum
{
    UVM_PERF_MODULE_TYPE_STATS = 0,
    UVM_PERF_MODULE_TYPE_THRASHING,
    UVM_PERF_MODULE_TYPE_PREFETCH,
    UVM_PERF_MODULE_TYPE_COUNT,
} uvm_perf_module_type_t;

// Array with pointers to module-defined data. va_space, va_range and va_block contain one field of this type.
// TODO: change a void pointer with a struct that contains some more information (module info, struct list_head)
typedef void *uvm_perf_modules_data_t[UVM_PERF_MODULE_TYPE_COUNT];

NV_STATUS uvm_perf_init(void);

void uvm_perf_exit(void);

NV_STATUS uvm_perf_get_thrashing_prevention(UVM_PERF_GET_THRASHING_PREVENTION_PARAMS *params, struct file *filp);

NV_STATUS uvm_perf_set_thrashing_prevention(UVM_PERF_SET_THRASHING_PREVENTION_PARAMS *params, struct file *filp);

// Thrashing prevention tunables via module_params
extern int g_uvm_nothrashing;
extern int g_uvm_thrashing_fault_count;
extern int g_uvm_thrashing_lapse;
extern int g_uvm_thrashing_nap;

#endif
